@@ -8,10 +8,12 @@ from irods.message import (
     DataObjChksumRequest, DataObjChksumResponse, RErrorStack)
 import irods.exception as ex
 from irods.api_number import api_number
+from irods.collection import iRODSCollection
 from irods.data_object import (
     iRODSDataObject, iRODSDataObjectFileRaw, chunks, irods_dirname, irods_basename)
 import irods.keywords as kw
 import irods.parallel as parallel
+from irods.parallel import deferred_call
 import six
 import ast
 import json
@@ -22,7 +24,7 @@ MAXIMUM_SINGLE_THREADED_TRANSFER_SIZE = 32 * ( 1024 ** 2)
 
 DEFAULT_NUMBER_OF_THREADS = 0   # Defaults for reasonable number of threads -- optimized to be
                                 # performant but allow no more worker threads than available CPUs.
-                                # Setting this to 1 disables automatic use of parallel transfer.
+
 DEFAULT_QUEUE_DEPTH = 32
 
 
@@ -52,7 +54,12 @@ class DataObjectManager(Manager):
     def should_parallelize_transfer( self,
                                      num_threads = 0,
                                      obj_sz = 1+MAXIMUM_SINGLE_THREADED_TRANSFER_SIZE,
-                                     server_version_hint = ()):
+                                     server_version_hint = (),
+                                     measured_obj_size = ()  ## output variable. If a list is provided, it shall
+                                                              # be truncated to contain one value, the size of the
+                                                              # seekable object (if one is provided for `obj_sz').
+        ):
+
         # Allow an environment variable to override the detection of the server version.
         # Example: $ export IRODS_VERSION_OVERRIDE="4,2,9" ;  python -m irods.parallel ...
         # ---
@@ -68,6 +75,7 @@ class DataObjectManager(Manager):
             if not isinstance(size,six.integer_types):
                 size = obj_sz.tell()
             obj_sz.seek(pos,os.SEEK_SET)
+            if isinstance(measured_obj_size,list): measured_obj_size[:] = [size]
         else:
             size = obj_sz
             assert (size > -1)
@@ -75,7 +83,10 @@ class DataObjectManager(Manager):
 
 
     def _download(self, obj, local_path, num_threads, **options):
+        """Transfer the contents of a data object to a local file.
 
+        Called from get() when a local path is named.
+        """
         if os.path.isdir(local_path):
             local_file = os.path.join(local_path, irods_basename(obj))
         else:
@@ -98,6 +109,12 @@ class DataObjectManager(Manager):
 
 
     def get(self, path, local_path = None, num_threads = DEFAULT_NUMBER_OF_THREADS, **options):
+        """
+        Get a reference to the data object at the specified `path'.
+
+        Only download the object if the local_path is a string (specifying
+        a path in the local filesystem to use as a destination file).
+        """
         parent = self.sess.collections.get(irods_dirname(path))
 
         # TODO: optimize
@@ -121,29 +138,37 @@ class DataObjectManager(Manager):
 
     def put(self, local_path, irods_path, return_data_object = False, num_threads = DEFAULT_NUMBER_OF_THREADS, **options):
 
-        if irods_path.endswith('/'):
-            obj = irods_path + os.path.basename(local_path)
+        if self.sess.collections.exists(irods_path):
+            obj = iRODSCollection.normalize_path(irods_path, os.path.basename(local_path))
         else:
             obj = irods_path
 
-        with open(local_path, 'rb') as f, self.open(obj, 'w', **options) as o:
-
-            if self.should_parallelize_transfer (num_threads, f):
+        with open(local_path, 'rb') as f:
+            sizelist = []
+            if self.should_parallelize_transfer (num_threads, f, measured_obj_size = sizelist):
+                o = deferred_call( self.open, (obj, 'w'), options)
                 f.close()
-                if not self.parallel_put( local_path, (obj,o), num_threads = num_threads,
+                if not self.parallel_put( local_path, (obj,o), total_bytes = sizelist[0], num_threads = num_threads,
                                           target_resource_name = options.get(kw.RESC_NAME_KW,'') or
-                                                                 options.get(kw.DEST_RESC_NAME_KW,'')):
+                                                                 options.get(kw.DEST_RESC_NAME_KW,''),
+                                          open_options = options ):
                     raise RuntimeError("parallel put failed")
             else:
-                # Set operation type to trigger acPostProcForPut
-                if kw.OPR_TYPE_KW not in options:
-                    options[kw.OPR_TYPE_KW] = 1 # PUT_OPR
-                for chunk in chunks(f, self.WRITE_BUFFER_SIZE):
-                    o.write(chunk)
-
+                with self.open(obj, 'w', **options) as o:
+                    # Set operation type to trigger acPostProcForPut
+                    if kw.OPR_TYPE_KW not in options:
+                        options[kw.OPR_TYPE_KW] = 1 # PUT_OPR
+                    for chunk in chunks(f, self.WRITE_BUFFER_SIZE):
+                        o.write(chunk)
         if kw.ALL_KW in options:
-            options[kw.UPDATE_REPL_KW] = ''
-            self.replicate(obj, **options)
+            repl_options = options.copy()
+            repl_options[kw.UPDATE_REPL_KW] = ''
+            # Leaving REG_CHKSUM_KW set would raise the error:
+            # Requested to register checksum without verifying, but source replica has a checksum. This can result
+            # in multiple replicas being marked good with different checksums, which is an inconsistency.
+            del repl_options[kw.REG_CHKSUM_KW]
+            self.replicate(obj, **repl_options)
+
 
         if return_data_object:
             return self.get(obj)
@@ -190,7 +215,13 @@ class DataObjectManager(Manager):
                      num_threads = 0,
                      target_resource_name = '',
                      progressQueue = False):
+        """Call into the irods.parallel library for multi-1247 GET.
 
+        Called from a session.data_objects.get(...) (via the _download method) on
+        the condition that the data object is determined to be of appropriate size
+        for parallel download.
+
+        """
         return parallel.io_main( self.sess, data_or_path_, parallel.Oper.GET | (parallel.Oper.NONBLOCKING if async_ else 0), file_,
                                  num_threads = num_threads, target_resource_name = target_resource_name,
                                  queueLength = (DEFAULT_QUEUE_DEPTH if progressQueue else 0))
@@ -199,13 +230,22 @@ class DataObjectManager(Manager):
                      file_ ,
                      data_or_path_ ,
                      async_ = False,
+                     total_bytes = -1,
                      num_threads = 0,
                      target_resource_name = '',
+                     open_options = {},
                      progressQueue = False):
+        """Call into the irods.parallel library for multi-1247 PUT.
 
+        Called from a session.data_objects.put(...) on the condition that the
+        data object is determined to be of appropriate size for parallel upload.
+
+        """
         return parallel.io_main( self.sess, data_or_path_, parallel.Oper.PUT | (parallel.Oper.NONBLOCKING if async_ else 0), file_,
-                                 num_threads = num_threads, target_resource_name = target_resource_name,
-                                 queueLength = (DEFAULT_QUEUE_DEPTH if progressQueue else 0))
+                                 num_threads = num_threads, total_bytes = total_bytes,  target_resource_name = target_resource_name,
+                                 open_options = open_options,
+                                 queueLength = (DEFAULT_QUEUE_DEPTH if progressQueue else 0)
+                               )
 
 
     def create(self, path, resource=None, force=False, **options):
@@ -250,9 +290,17 @@ class DataObjectManager(Manager):
         handle = self.open(*arg,_raw_fd_holder=holder,**kw_options)
         return (handle, holder[-1])
 
+    _RESC_flags_for_open = frozenset((
+                               kw.RESC_NAME_KW,
+                               kw.DEST_RESC_NAME_KW,  # may be deprecated in the future
+                               kw.RESC_HIER_STR_KW
+                           ))
+
     def open(self, path, mode, create = True, finalize_on_close = True, **options):
         _raw_fd_holder =  options.get('_raw_fd_holder',[])
-        if kw.DEST_RESC_NAME_KW not in options:
+        # If no keywords are used that would influence the server as to the choice of a storage resource,
+        # then use the default resource in the client configuration.
+        if self._RESC_flags_for_open.isdisjoint( options.keys() ):
             # Use client-side default resource if available
             try:
                 options[kw.DEST_RESC_NAME_KW] = self.sess.default_resource
@@ -296,6 +344,25 @@ class DataObjectManager(Manager):
         return io.BufferedRandom(raw)
 
 
+    def trim(self, path, **options):
+
+        message_body = FileOpenRequest(
+            objPath=path,
+            createMode=0,
+            openFlags=0,
+            offset=0,
+            dataSize=-1,
+            numThreads=self.sess.numThreads,
+            KeyValPair_PI=StringStringMap(options),
+        )
+        message = iRODSMessage('RODS_API_REQ', msg=message_body,
+                               int_info=api_number['DATA_OBJ_TRIM_AN'])
+
+        with self.sess.pool.get_connection() as conn:
+            conn.send(message)
+            response = conn.recv()
+
+
     def unlink(self, path, force=False, **options):
         if force:
             options[kw.FORCE_FLAG_KW] = ''
